@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2001-2002 Hewlett-Packard Co.
+ *  Copyright (C) 2001-2003 Hewlett-Packard Co.
  *	Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *
  * This file is part of the ELILO, the EFI Linux boot loader.
@@ -121,40 +121,46 @@ static EFI_GUID Ext2FsProtocol = EXT2FS_PROTOCOL;
 static INTN
 read_bytes(EFI_BLOCK_IO *blkio, UINT32 mediaid, UINTN offset, VOID *addr, UINTN size)
 {
-#define BLK_COUNT 32
 	EFI_STATUS status;
-	UINT8 buffer[EXT2_MIN_BLOCK_SIZE*BLK_COUNT]; /* XXX not good ! */
-	UINTN count;
+	UINT8 *buffer;
+	UINTN count, buffer_size;
 	EFI_LBA base;
+	INTN ret  = EFI_INVALID_PARAMETER;
 
-	if (size >sizeof(buffer)) {
-		ERR_PRT((L"requesting size %d>%d media blocksize", size, sizeof(buffer),blkio->Media->BlockSize));
-		return EFI_INVALID_PARAMETER;
-	}
-	base  = offset / blkio->Media->BlockSize;
-	count = (size + blkio->Media->BlockSize -1) / blkio->Media->BlockSize;
+	base        = offset / blkio->Media->BlockSize;
+	count       = (size + blkio->Media->BlockSize -1) / blkio->Media->BlockSize;
+	buffer_size = count * blkio->Media->BlockSize;
 
 	DBG_PRT((L"readblock(%x, %d,%d) base=%d count=%d", blkio, offset, size, base, count));
 
-	if (count > BLK_COUNT) {
-		ERR_PRT((L"requesting count %d", count));
-		return EFI_INVALID_PARAMETER;
+	/*
+	 * slow but avoid large buffer on the stack
+	 */
+	buffer = (UINT8 *)alloc(buffer_size, EfiLoaderData);
+	if (buffer == NULL) {
+		ERR_PRT((L"cannot allocate ext2fs buffer size=%d", buffer_size));
+		return ret;
 	}
 
-	DBG_PRT((L"readblock(%x, %d, %d, %d, %x)", blkio, mediaid, base, count *blkio->Media->BlockSize, buffer));
+	DBG_PRT((L"readblock(%x, %d, %d, %d, %x)", blkio, mediaid, base, buffer_size, buffer));
 
-	status = blkio->ReadBlocks(blkio, mediaid, base, count *blkio->Media->BlockSize, buffer); 
+	status = uefi_call_wrapper(blkio->ReadBlocks, 5, blkio, mediaid, base, buffer_size, buffer); 
 	if (EFI_ERROR(status)) {
-		ERR_PRT((L"readblock(%d,%d)=%r", base, count, status));
-		return EFI_INVALID_PARAMETER;
+		ERR_PRT((L"readblock(%d,%d)=%r", base, buffer_size, status));
+		goto error;
 	}
-	DBG_PRT((L"readblock(%d,%d)->%r", offset, size, status));
+
+	DBG_PRT((L"readblock(%d,%d)->%r", offset, buffer_size, status));
 
 	Memcpy(addr, buffer+(offset-(base*blkio->Media->BlockSize)), size);
 
-	return 0;
-}
+	ret = 0;
 
+error:
+	free(buffer);
+
+	return ret;
+}
 
 /*
  * Read the specified inode from the disk and return it to the user.
@@ -791,7 +797,6 @@ ext2fs_init_state(ext2fs_t *ext2fs, EFI_HANDLE dev, EFI_BLOCK_IO *blkio, struct
 	e2fs->gds = (struct ext2_group_desc *)alloc(e2fs->ngroups * sizeof(struct ext2_group_desc), EXT2FS_MEMTYPE);
 	if (e2fs->gds == NULL) {
 		ERR_PRT((L"failed to allocate gds"));
-		free(ext2fs);
 		return EFI_OUT_OF_RESOURCES;
 	}
 	
@@ -808,7 +813,6 @@ ext2fs_init_state(ext2fs_t *ext2fs, EFI_HANDLE dev, EFI_BLOCK_IO *blkio, struct
 	if (EFI_ERROR(status)) {
 		ERR_PRT((L"cannot read gds: %r", status));
 		free(e2fs->gds);
-		free(ext2fs);
 		return EFI_INVALID_PARAMETER;
 	}
 #if 0
@@ -862,13 +866,13 @@ ext2fs_install_one(EFI_HANDLE dev, VOID **intf)
 	EFI_BLOCK_IO *blkio;
 	ext2fs_t *ext2fs;
 
-	status = BS->HandleProtocol (dev, &Ext2FsProtocol, (VOID **)&ext2fs);
+	status = uefi_call_wrapper(BS->HandleProtocol, 3, dev, &Ext2FsProtocol, (VOID **)&ext2fs);
 	if (status == EFI_SUCCESS) {
 		ERR_PRT((L"Warning: found existing %s protocol on device", FS_NAME));
 		goto found;
 	}
 	
-	status = BS->HandleProtocol(dev, &BlockIoProtocol, (VOID **)&blkio);
+	status = uefi_call_wrapper(BS->HandleProtocol, 3, dev, &BlockIoProtocol, (VOID **)&blkio);
 	if (EFI_ERROR(status)) return EFI_INVALID_PARAMETER;
 	
 	VERB_PRT(5,
@@ -940,7 +944,7 @@ ext2fs_install(VOID)
 	EFI_STATUS status;
 	VOID *intf;
 
-	BS->LocateHandle(ByProtocol, &BlockIoProtocol, NULL, &size, NULL);
+	uefi_call_wrapper(BS->LocateHandle, 5, ByProtocol, &BlockIoProtocol, NULL, &size, NULL);
 	if (size == 0) return EFI_UNSUPPORTED; /* no device found, oh well */
 
 	DBG_PRT((L"size=%d", size));
@@ -951,7 +955,8 @@ ext2fs_install(VOID)
 		return EFI_OUT_OF_RESOURCES;
 	}
 	
-	status = BS->LocateHandle(ByProtocol, &BlockIoProtocol, NULL, &size, (VOID **)dev_tab);
+	status = uefi_call_wrapper(BS->LocateHandle, 5, ByProtocol, &BlockIoProtocol, NULL, 
+				&size, (VOID **)dev_tab);
 	if (status != EFI_SUCCESS) {
 		ERR_PRT((L"failed to get handles: %r", status));
 		free(dev_tab);
@@ -980,7 +985,7 @@ ext2fs_uninstall(VOID)
 	for(i=0; i < ndev; i++) {
 		if (dev_tab[i].intf == NULL) continue;
 		e2fs = FS_PRIVATE(dev_tab[i].intf);
-		status = BS->UninstallProtocolInterface(e2fs->dev, &Ext2FsProtocol, dev_tab[i].intf);
+		status = uefi_call_wrapper(BS->UninstallProtocolInterface, 3, e2fs->dev, &Ext2FsProtocol, dev_tab[i].intf);
 		if (EFI_ERROR(status)) {
 			ERR_PRT((L"Uninstall %s error: %r", FS_NAME, status));
 			continue;
@@ -994,7 +999,7 @@ ext2fs_uninstall(VOID)
 			});
 		free(dev_tab[i].intf);
 	}
-	free(dev_tab);
+	if (dev_tab) free(dev_tab);
 
 	return EFI_SUCCESS;
 }
